function [Abar,Fbar,B1bar,B2bar,Cbar,Pvbar,W,Gamstarbar,Cstarbar,Pvstarbar,Pwstarbar,messbar,Gamstar,Cstar,Pvstar,Pwstar, mess,lams,lamt,lamsbar,lamtbar]=FindEquivMD(A,F,B1,B2,C,Pv,R,r,V)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is the workhorse program for the paper "Identification in Rational
% Expectations Models" (February 2004) by Andreas Beyer and Roger E A Farmer.
% The code takes as input a rational expectations model described by a set of
% matrices A F B1 B2 C and Pv where 
%
% A*X(t) + F*Et[X(t+1)] = B1*X(t-1) + B2*Et-1[X(t)] + Pv*v(t) + C
%
% and a set of restriction R,r where R*vec([Abar Fbar -B1bar -B2bar Cbar Pvbar]')=r
% exactly identifies some alternative model.  The original model is assumed
% to be determinate. 
%
% The argument V is optional. If it is missing it is taken to be the
% identity matrx.
%
% The output is an EQUIVALENT system of the form
%
% Abar*X(t) + Fbar*Et[X(t+1)] = B1bar*X(t-1) + B2bar*Et-1[X(t)] +Pvbar*v1a(t) + Cbar
%
% The equivalent system satisfies the restrictions 
%
% R*vec([Abar Fbar -B1bar -B2bar Cbar Pvbar]')=r
%
% The equivalent system may be indeterminate.
% The code  returns the reduced form of both systems where
% 
% X(t)= Gamstar*X(t-1) + Cstar + Pvstar*v(t)  
% for the original system
%
% X(t)= Gamstarbar*X(t-1) + Cstarbar + [Pvstarbar Pwstarbar]*v1(t) where v1(t)=[v1a(t) w(t)] 
% for the equivalent system
% 
% If V is the VCV matrix of v(t) then W is the vcv matrix of [v1a(t); w(t)] 
% where w(t) represents sunspot shocks.  The default is that V is the identity matrix. 
% The matrices V and W are related by the condition
% Pvstar*V*Pvstar' = [Pvstarbar Pwstarbar]*W*[Pvstarbar Pwstarbar]'
% 
% If the matrices Gamstar and Gamstarbar are not identical, there is a
% reordering of variables for each system that makes them equivalent.  
% To find the equivalent reordering- run the program convert.m to rearrange each system.
%
% mess and messbar are text messages regarding the determinacy of the original
% and the equivalent system
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Andreas Beyer and Roger E. A. Farmer
% February 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin == 8 V=eye(size(A,1)); else end;
        
n=size(A,1);
Astar0=[A F;eye(n) zeros(n)];
Astar1=[B1 B2;zeros(n) eye(n)];
Cstar0=[C;zeros(n,1)];
Pvstar0=[Pv;zeros(n)];
Pwstar0=[zeros(n);eye(n)];
[Gamstar,Cstar,Pvstar,Pwstar, mess,lams,lamt]=SysSolveMD(Astar0,Astar1,Cstar0,Pvstar0,Pwstar0);

E=[Gamstar';Cstar';Pvstar'];
G=[E -eye(size(E,1))];
H=kron(eye(n),G);
J=[H ; R];
Q=[zeros(size(H,1),1);r];
Ltilde=inv(J)*Q;
M=reshape(Ltilde,size(J,1)/n,n);
M=M';
Abar=M(:,1:n);
Fbar=M(:,n+1:2*n);
B1bar=M(:,2*n+1:3*n);
B2bar=M(:,3*n+1:4*n);
Cbar=M(:,4*n+1);
Pvbar=M(:,4*n+2:4*n+1+size(Pvstar0,2));

Astar0bar=[Abar Fbar;eye(n) zeros(n)];
Astar1bar=[B1bar B2bar;zeros(n) eye(n)];
Cstar0bar=[Cbar;zeros(n,1)];
Pvstar0bar=[Pvbar;zeros(n)];
Pwstar0bar=[zeros(n);eye(n)];
[Gamstarbar,Cstarbar,Pvstarbar,Pwstarbar, messbar,lamsbar,lamtbar]=SysSolveMD(Astar0bar,Astar1bar,Cstar0bar,Pvstar0bar,Pwstar0bar);

A=[Pvstar(1:n,:) ];B=[Pvstarbar(1:n,:) Pwstarbar(1:n,:)];W=VCVsolve(A,B,V);



